#!/usr/bin/python3
# -*- coding: utf-8 -*-

#/*
# * Copyright (C) 2020, KylinSoft Co., Ltd.
# *
# * This program is free software: you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation, either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <https://www.gnu.org/licenses/>.
# */


import os
import sys


import logging
from logging import handlers
from xdg import BaseDirectory as xdg

userCacheUkscDir = os.path.join(xdg.xdg_cache_home, "uksc")
if not os.path.exists(userCacheUkscDir):
        try:
            os.makedirs(userCacheUkscDir)
        except OSError as e:
            pass
userCacheDirLog = os.path.join(userCacheUkscDir,"log")

if not os.path.exists(userCacheDirLog):
        try:
            os.makedirs(userCacheDirLog)
        except OSError as e:
            pass

infoLog = os.path.join(userCacheDirLog, "kylin-software-center-plugin-preprocessing-info.log")
errorLog = os.path.join(userCacheDirLog, "kylin-software-center-plugin-preprocessing-error.log")
DEBUG_LOG = os.path.join(userCacheDirLog,"kylin-software-center-debug.log")

class LogManager(object):
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'crit': logging.CRITICAL
    }  # 日志级别关系映射

    def __init__(self, filename, level='info', when="W0", backCount=1,
                 fmt='%(asctime)s - %(pathname)s[line:%(lineno)d] - %(levelname)s: %(message)s'):
        error_log_file = errorLog.split("/")[len(errorLog.split("/"))-1]
        info_log_file = infoLog.split("/")[len(infoLog.split("/")) - 1]
        debug_log_file = DEBUG_LOG.split("/")[len(DEBUG_LOG.split("/")) - 1]
        if(os.path.exists(userCacheDirLog)):
            for file in os.listdir(userCacheDirLog):
                if(file != info_log_file and file != error_log_file and file != debug_log_file):
                    os.remove(os.path.join(userCacheDirLog,file))
        self.logger = logging.getLogger(filename)
        format_str = logging.Formatter(fmt)  # 设置日志格式
        self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
        # sh = logging.StreamHandler()  # 往屏幕上输出
        # sh.setFormatter(format_str)  # 设置屏幕上显示的格式
        th = handlers.TimedRotatingFileHandler(filename=filename, when=when, backupCount=backCount,
                                               encoding='utf-8')  # 往文件里写入#指定间隔时间自动生成文件的处理器
        # 实例化TimedRotatingFileHandler
        # interval是时间间隔，backupCount是备份文件的个数，如果超过这个个数，就会自动删除，when是间隔的时间单位，单位有以下几种：
        # S 秒
        # M 分
        # H 小时、
        # D 天、
        # W 每星期（interval==0时代表星期一）
        # midnight 每天凌晨
        th.setFormatter(format_str)  # 设置文件里写入的格式
        # self.logger.addHandler(sh)  # 把对象加到logger里
        self.logger.addHandler(th)
